@@ -248,6 +248,17 @@
 	my $dbh = shift; my $re = shift;
 	grep { $_ =~ /$re/ } $dbh->tables();
     }
+
+    sub parse_trace_flag {
+	my ($h, $name) = @_;
+	return 0x01000000 if $name eq 'foo';
+	return 0x02000000 if $name eq 'bar';
+	return 0x04000000 if $name eq 'baz';
+	return 0x08000000 if $name eq 'boo';
+	return 0x10000000 if $name eq 'bop';
+	return $h->SUPER::parse_trace_flag($name);
+    }
+
 }
 
 
@@ -383,6 +394,9 @@
     }
 
     sub DESTROY { undef }
+
+    *parse_trace_flag = \&DBD::ExampleP::db::parse_trace_flag;
 }
 
 1;
+# vim: sw=4:ts=8
